/*********************************************************************************************************
 * EncoderOdometry.ino
 * Calculates the x & y positions and the angle of the robot and displays it on TeensyView once every 10ms
 * Reference: Implementing Dead Reckoning by Odometry on a Robot with R/C Servo Differential Drive
 * http://www.seattlerobotics.org/encoder/200010/dead_reckoning_article.html
 */

#include <Encoder.h>
#include <TeensyView.h>

#define PIN_RESET 15
#define PIN_DC    21
#define PIN_CS    20
#define PIN_SCK   13
#define PIN_MOSI  11

const double WHEEL_DIA = 3.70;
const double DISTANCE_PER_COUNT = 0.04843288674284264575; //PI*WHEEL_DIA/240
const double WHEEL_SPACING = 15.00;

Encoder left(4,0);
Encoder right(9,27);
TeensyView oled(PIN_RESET, PIN_DC, PIN_CS, PIN_SCK, PIN_MOSI);

double leftDistance, rightDistance, totalDistance=0.0, expr, x=0.0, y=0.0, theta=0.0;
int leftTicks, rightTicks;

void setup()
{
  oled.begin();
  oled.clear(PAGE);
  oled.flipVertical(1);
  oled.flipHorizontal(1);
  oled.display();
}

void loop()
{
  leftTicks = left.read();
  rightTicks = right.read();
  left.write(0);
  right.write(0);
  leftDistance = (double)leftTicks*DISTANCE_PER_COUNT;
  rightDistance = (double)rightTicks*DISTANCE_PER_COUNT;
  totalDistance += (leftDistance + rightDistance)/2;
  
  if(leftTicks == rightTicks)
  {
    x += leftDistance*cos(theta);
    y += rightDistance*sin(theta);
  }
  else
  {
    expr = WHEEL_SPACING*(rightDistance+leftDistance)/2/(rightDistance-leftDistance);
    x += expr*(sin((rightDistance-leftDistance)/WHEEL_SPACING+theta)-sin(theta));
    y -= expr*(cos((rightDistance-leftDistance)/WHEEL_SPACING+theta)-cos(theta));

    theta += (rightDistance-leftDistance)/WHEEL_SPACING;

    while(theta > PI)
      theta -= 2*PI;
    while(theta < -PI)
      theta += 2*PI;
  }

  oled.clear(PAGE);
  oled.setCursor(0,0);
  oled.print("(");
  oled.print(x);
  oled.print(",");
  oled.print(y);
  oled.print(")");
  oled.setCursor(0,10);
  oled.print("Distance = ");
  oled.print(totalDistance);
  oled.print("cm");
  oled.setCursor(0,20);
  oled.print("Angle = ");
  oled.print(theta*180/PI);
  oled.display();
}
